"use strict";
var RelativeDateScalar_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.RelativeDateScalar = exports.isRelativeDateScalar = exports.parseRelativeDateFormat = void 0;
const tslib_1 = require("tslib");
const graphql_1 = require("@nestjs/graphql");
const graphql_2 = require("graphql");
const date_fns_1 = require("date-fns");
var RelativeDateResolution;
(function (RelativeDateResolution) {
    RelativeDateResolution["minutes"] = "minutes";
    RelativeDateResolution["hours"] = "hours";
    RelativeDateResolution["days"] = "days";
    RelativeDateResolution["weeks"] = "weeks";
    RelativeDateResolution["months"] = "months";
    RelativeDateResolution["years"] = "years";
})(RelativeDateResolution || (RelativeDateResolution = {}));
const RelativeDatePrefix = 'last';
const ACCEPTED_VALUE = 'an Epoch time (number), ISO Date string or a relative date value in the form of "last-[N]-[minutes | hours | days | weeks | months | years]"';
const INVALID_FORMAT_ERROR = () => {
    throw new Error(`Invalid relative date value! Can only accept ${ACCEPTED_VALUE}`);
};
const parseRelativeDateFormat = (value) => {
    let span;
    const [prefix, spanStr, resolution] = value.split('-');
    if (prefix !== RelativeDatePrefix)
        INVALID_FORMAT_ERROR();
    try {
        span = parseInt(spanStr, 10);
    }
    catch (e) {
        INVALID_FORMAT_ERROR();
    }
    if (!RelativeDateResolution[resolution])
        INVALID_FORMAT_ERROR();
    return {
        resolution,
        span,
    };
};
exports.parseRelativeDateFormat = parseRelativeDateFormat;
const getRelativeDate = (value) => {
    const { span, resolution } = (0, exports.parseRelativeDateFormat)(value);
    return (0, date_fns_1.sub)(new Date(), {
        [resolution]: span,
    });
};
const isRelativeDateScalar = (value) => value.startsWith(RelativeDatePrefix);
exports.isRelativeDateScalar = isRelativeDateScalar;
const isIsoDate = (str) => {
    if (!/\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}.\d{3}Z/.test(str))
        return false;
    const d = new Date(str);
    return d instanceof Date && !isNaN(d.getTime()) && d.toISOString() === str; // valid date
};
let RelativeDateScalar = RelativeDateScalar_1 = class RelativeDateScalar {
    constructor() {
        this.description = `Relative date, ${ACCEPTED_VALUE}`;
    }
    parseValue(value) {
        if (typeof value === 'number')
            return new Date(value);
        if ((0, exports.isRelativeDateScalar)(value))
            return getRelativeDate(value);
        if (isIsoDate(value))
            return new Date(value);
        return INVALID_FORMAT_ERROR();
    }
    serialize(value) {
        if (value instanceof Date)
            return value.toISOString();
        return new Date(value).toISOString();
    }
    parseLiteral(ast) {
        if (ast.kind === graphql_2.Kind.INT) {
            return new Date(ast.value);
        }
        else if (ast.kind === graphql_2.Kind.STRING) {
            if ((0, exports.isRelativeDateScalar)(ast.value))
                return getRelativeDate(ast.value);
            if (isIsoDate(ast.value))
                return new Date(ast.value);
        }
        return null;
    }
};
RelativeDateScalar = RelativeDateScalar_1 = tslib_1.__decorate([
    (0, graphql_1.Scalar)('RelativeDate', () => RelativeDateScalar_1)
], RelativeDateScalar);
exports.RelativeDateScalar = RelativeDateScalar;
//# sourceMappingURL=relative-date-scalar.type.js.map